@@ -20,13 +20,19 @@ namespace py = pybind11;
 
 
 class ParticleHelper {
+public:
+
+    typedef Kokkos::View<double***, Kokkos::HostSpace> double_3d_view_type;
+    typedef Kokkos::View<double**, Kokkos::HostSpace> double_2d_view_type;
+    typedef Kokkos::View<int**, Kokkos::HostSpace> int_2d_view_type;
+    typedef Kokkos::View<double*, Kokkos::HostSpace> double_1d_view_type;
+    typedef Kokkos::View<int*, Kokkos::HostSpace> int_1d_view_type;
+    typedef Kokkos::View<int*> int_1d_view_type_in_gmls;
 
 private:
 
     Compadre::GMLS* gmls_object;
-    typedef Kokkos::View<double**, Kokkos::HostSpace> double_2d_view_type;
-    typedef Kokkos::View<int**, Kokkos::HostSpace> int_2d_view_type;
-    typedef Kokkos::View<double*, Kokkos::HostSpace> double_1d_view_type;
+    Compadre::NeighborLists<int_1d_view_type_in_gmls>* nl;
 
     typedef nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<double, Compadre::PointCloudSearch<double_2d_view_type> >, 
             Compadre::PointCloudSearch<double_2d_view_type>, 3> tree_type;
@@ -34,9 +40,12 @@ private:
 
     double_2d_view_type _source_coords;
     double_2d_view_type _target_coords;
-    int_2d_view_type _neighbor_lists;
     double_1d_view_type _epsilon;
 
+    // optional
+    double_3d_view_type _tangent_bundle;
+    double_2d_view_type _reference_normal_directions;
+
 public:
 
     ParticleHelper(GMLS& gmls_instance) {
@@ -65,26 +74,12 @@ public:
     
         // set values from Kokkos View
         gmls_object->setNeighborLists(neighbor_lists);
-        _neighbor_lists = neighbor_lists;
+        nl = gmls_object->getNeighborLists();
     }
 
-    py::array_t<local_index_type> getNeighbors() {
-        compadre_assert_release((_neighbor_lists.extent(0)>0) && "getNeighborLists() called, but neighbor lists were never set.");
-
-        auto dim_out_0 = _neighbor_lists.extent(0);
-        auto dim_out_1 = _neighbor_lists.extent(1);
-
-        auto result = py::array_t<local_index_type>(dim_out_0*dim_out_1);
-        py::buffer_info buf_out = result.request();
-
-        local_index_type *ptr = (local_index_type *) buf_out.ptr;
-        Kokkos::parallel_for(Kokkos::RangePolicy<Kokkos::DefaultHostExecutionSpace>(0,dim_out_0*dim_out_1), [&](int i) {
-            ptr[i] = *(_neighbor_lists.data()+i);
-        });
-        Kokkos::fence();
-
-        result.resize({dim_out_0,dim_out_1});
-        return result;
+    decltype(nl) getNeighborLists() {
+        compadre_assert_release((nl->getNumberOfTargets()>0) && "getNeighborLists() called, but neighbor lists were never set.");
+        return gmls_object->getNeighborLists();
     }
 
     void setSourceSites(py::array_t<double> input) {
@@ -222,6 +217,109 @@ public:
         return result;
     }
 
+    void setTangentBundle(py::array_t<double> input) {
+        py::buffer_info buf = input.request();
+
+        if (buf.ndim != 3) {
+            throw std::runtime_error("Number of dimensions must be three");
+        }
+
+        if (gmls_object->getGlobalDimensions()!=input.shape(1)) {
+            throw std::runtime_error("Second dimension must be the same as GMLS spatial dimension");
+        }
+
+        if (gmls_object->getGlobalDimensions()!=input.shape(2)) {
+            throw std::runtime_error("Second dimension must be the same as GMLS spatial dimension");
+        }
+        
+        // create Kokkos View on host to copy into
+        Kokkos::View<double***, Kokkos::HostSpace> tangent_bundle("tangent bundle", input.shape(0), input.shape(1), input.shape(2));
+        
+        // overwrite existing data assuming a 2D layout
+        auto data = input.unchecked<3>();
+        Kokkos::parallel_for(Kokkos::RangePolicy<Kokkos::DefaultHostExecutionSpace>(0,input.shape(0)), [=](int i) {
+            for (int j = 0; j < input.shape(1); ++j)
+            {
+                for (int k = 0; k < input.shape(2); ++k) {
+                    tangent_bundle(i, j, k) = data(i, j, k);
+                }
+            }
+        });
+        Kokkos::fence();
+        
+        // set values from Kokkos View
+        gmls_object->setTangentBundle(tangent_bundle);
+        _tangent_bundle = tangent_bundle;
+    }
+
+    py::array_t<double> getTangentBundle() {
+        compadre_assert_release((_tangent_bundle.extent(0)>0) && "getTangentBundle() called, but tangent bundle was never set.");
+
+        auto dim_out_0 = _tangent_bundle.extent(0);
+        auto dim_out_1 = _tangent_bundle.extent(1);
+        auto dim_out_2 = _tangent_bundle.extent(2);
+
+        auto result = py::array_t<double>(dim_out_0*dim_out_1*dim_out_2);
+        py::buffer_info buf_out = result.request();
+
+        double *ptr = (double *) buf_out.ptr;
+        Kokkos::parallel_for(Kokkos::RangePolicy<Kokkos::DefaultHostExecutionSpace>(0,dim_out_0*dim_out_1*dim_out_2), [&](int i) {
+            ptr[i] = *(_tangent_bundle.data()+i);
+        });
+        Kokkos::fence();
+
+        result.resize({dim_out_0,dim_out_1,dim_out_2});
+        return result;
+    }
+
+    void setReferenceOutwardNormalDirection(py::array_t<double> input, bool use_to_orient_surface = true) {
+        py::buffer_info buf = input.request();
+
+        if (buf.ndim != 2) {
+            throw std::runtime_error("Number of dimensions must be two");
+        }
+
+        if (gmls_object->getGlobalDimensions()!=input.shape(1)) {
+            throw std::runtime_error("Second dimension must be the same as GMLS spatial dimension");
+        }
+        
+        // create Kokkos View on host to copy into
+        Kokkos::View<double**, Kokkos::HostSpace> reference_normal_directions("reference normal directions", input.shape(0), input.shape(1));
+        
+        // overwrite existing data assuming a 2D layout
+        auto data = input.unchecked<2>();
+        Kokkos::parallel_for(Kokkos::RangePolicy<Kokkos::DefaultHostExecutionSpace>(0,input.shape(0)), [=](int i) {
+            for (int j = 0; j < input.shape(1); ++j)
+            {
+                reference_normal_directions(i, j) = data(i, j);
+            }
+        });
+        Kokkos::fence();
+        
+        // set values from Kokkos View
+        gmls_object->setReferenceOutwardNormalDirection(reference_normal_directions, true /* use to orient surface*/);
+        _reference_normal_directions = reference_normal_directions;
+    }
+
+    py::array_t<double> getReferenceOutwardNormalDirection() {
+        compadre_assert_release((_reference_normal_directions.extent(0)>0) && "getReferenceNormalDirections() called, but normal directions were never set.");
+
+        auto dim_out_0 = _reference_normal_directions.extent(0);
+        auto dim_out_1 = _reference_normal_directions.extent(1);
+
+        auto result = py::array_t<double>(dim_out_0*dim_out_1);
+        py::buffer_info buf_out = result.request();
+
+        double *ptr = (double *) buf_out.ptr;
+        Kokkos::parallel_for(Kokkos::RangePolicy<Kokkos::DefaultHostExecutionSpace>(0,dim_out_0*dim_out_1), [&](int i) {
+            ptr[i] = *(_reference_normal_directions.data()+i);
+        });
+        Kokkos::fence();
+
+        result.resize({dim_out_0,dim_out_1});
+        return result;
+    }
+
     void generateKDTree(py::array_t<double> input) {
         py::buffer_info buf = input.request();
 
@@ -250,7 +348,7 @@ public:
         gmls_object->setSourceSites(source_coords);
     }
 
-    void generateNeighborListsFromKNNSearchAndSet(py::array_t<double> input, int poly_order, int dimension = 3, double epsilon_multiplier = 1.6, double maximum_neighbors_storage_multiplier = 1.0, double max_search_radius = 0.0) {
+    void generateNeighborListsFromKNNSearchAndSet(py::array_t<double> input, int poly_order, int dimension = 3, double epsilon_multiplier = 1.6, double max_search_radius = 0.0) {
 
         int neighbors_needed = Compadre::GMLS::getNP(poly_order, dimension);
 
@@ -279,32 +377,34 @@ public:
         // how many target sites
         int number_target_coords = target_coords.extent(0);
 
-        // estimate storage requirements, but if storage_multiplier <= 0, then allocate as many entries as there are neighbors
-        int estimated_upper_bound_number_neighbors = (maximum_neighbors_storage_multiplier > 0) ?
-            (int)(maximum_neighbors_storage_multiplier*point_cloud_search->getEstimatedNumberNeighborsUpperBound(neighbors_needed, dimension, epsilon_multiplier))
-            : _source_coords.extent(0)+1; // +1 for # of neighbors value
+        // make empty neighbor list kokkos view
+        int_1d_view_type neighbor_lists("neighbor lists", 0);
         
-        // make neighbor list kokkos view
-        int_2d_view_type neighbor_lists("neighbor lists", 
-                number_target_coords, estimated_upper_bound_number_neighbors); // first column is # of neighbors
+        // holds number of neighbors for each target
+        int_1d_view_type number_of_neighbors_list("number of neighbors list", number_target_coords);
         
         // make epsilons kokkos view
         double_1d_view_type epsilon("h supports", number_target_coords);
         
         // call point_cloud_search using targets
         // use these neighbor lists and epsilons to set the gmls object
-        point_cloud_search->generateNeighborListsFromKNNSearch(false /* not a dry run*/, target_coords, neighbor_lists, 
-                epsilon, neighbors_needed, epsilon_multiplier, max_search_radius);
+        size_t total_storage = point_cloud_search->generateCRNeighborListsFromKNNSearch(true /* is a dry run*/, target_coords, neighbor_lists, 
+                number_of_neighbors_list, epsilon, neighbors_needed, epsilon_multiplier, max_search_radius);
 
+        Kokkos::resize(neighbor_lists, total_storage);
+        Kokkos::fence();
+
+        total_storage = point_cloud_search->generateCRNeighborListsFromKNNSearch(false /* not a dry run*/, target_coords, neighbor_lists, 
+                number_of_neighbors_list, epsilon, neighbors_needed, epsilon_multiplier, max_search_radius);
         Kokkos::fence();
 
         // set these views in the GMLS object
         gmls_object->setTargetSites(target_coords);
-        gmls_object->setNeighborLists(neighbor_lists);
+        gmls_object->setNeighborLists(neighbor_lists, number_of_neighbors_list);
         gmls_object->setWindowSizes(epsilon);
+        nl = gmls_object->getNeighborLists();
 
         _target_coords = target_coords;
-        _neighbor_lists = neighbor_lists;
         _epsilon = epsilon;
 
     }
@@ -337,7 +437,7 @@ public:
         // get polynomial coefficient size
         const int NP = gmls_object->getPolynomialCoefficientsSize();
         // get number of target sites
-        const int NT = gmls_object->getNeighborLists().extent(0);
+        const int NT = gmls_object->getNeighborLists()->getNumberOfTargets();
 
         Compadre::Evaluator gmls_evaluator(gmls_object);
         auto polynomial_coefficients = gmls_evaluator.applyFullPolynomialCoefficientsBasisToDataAllComponents<double**, Kokkos::HostSpace>
@@ -511,15 +611,20 @@ PYBIND11_MODULE(pycompadre, m) {
     .def("generateKDTree", &ParticleHelper::generateKDTree)
     .def("generateNeighborListsFromKNNSearchAndSet", &ParticleHelper::generateNeighborListsFromKNNSearchAndSet, 
             py::arg("target_sites"), py::arg("poly_order"), py::arg("dimension") = 3, py::arg("epsilon_multiplier") = 1.6, 
-            py::arg("maximum_neighbors_storage_multiplier") = 1.0, py::arg("max_search_radius") = 0.0)
-    .def("setNeighbors", &ParticleHelper::setNeighbors, py::arg("neighbor_lists"))
-    .def("getNeighbors", &ParticleHelper::getNeighbors, py::return_value_policy::take_ownership)
+            py::arg("max_search_radius") = 0.0)
+    .def("setNeighbors", &ParticleHelper::setNeighbors, py::arg("neighbor_lists"), 
+            "Sets neighbor lists from 2D array where first column is number of neighbors for corresponding row's target site.")
+    .def("getNeighborLists", &ParticleHelper::getNeighborLists, py::return_value_policy::reference_internal)
     .def("setWindowSizes", &ParticleHelper::setWindowSizes, py::arg("window_sizes"))
     .def("getWindowSizes", &ParticleHelper::getWindowSizes, py::return_value_policy::take_ownership)
     .def("setSourceSites", &ParticleHelper::setSourceSites, py::arg("source_coordinates"))
     .def("getSourceSites", &ParticleHelper::getSourceSites, py::return_value_policy::take_ownership)
     .def("setTargetSites", &ParticleHelper::setTargetSites, py::arg("target_coordinates"))
     .def("getTargetSites", &ParticleHelper::getTargetSites, py::return_value_policy::take_ownership)
+    .def("setTangentBundle", &ParticleHelper::setTangentBundle, py::arg("tangent_bundle"))
+    .def("getTangentBundle", &ParticleHelper::getTangentBundle, py::return_value_policy::take_ownership)
+    .def("setReferenceOutwardNormalDirection", &ParticleHelper::setReferenceOutwardNormalDirection, py::arg("reference_normal_directions"), py::arg("use_to_orient_surface") = true)
+    .def("getReferenceOutwardNormalDirection", &ParticleHelper::getReferenceOutwardNormalDirection, py::return_value_policy::take_ownership)
     .def("getPolynomialCoefficients", &ParticleHelper::getPolynomialCoefficients, py::arg("input_data"), py::return_value_policy::take_ownership)
     .def("applyStencil", &ParticleHelper::applyStencil, py::arg("input_data"), py::arg("target_operation")=TargetOperation::ScalarPointEvaluation, py::return_value_policy::take_ownership);
     
@@ -543,6 +648,12 @@ PYBIND11_MODULE(pycompadre, m) {
     .def("getNP", &GMLS::getNP, "Get size of basis.")
     .def("getNN", &GMLS::getNN, "Heuristic number of neighbors.");
 
+    py::class_<NeighborLists<ParticleHelper::int_1d_view_type_in_gmls> >(m, "NeighborLists")
+    .def("getNumberOfTargets", &NeighborLists<ParticleHelper::int_1d_view_type_in_gmls>::getNumberOfTargets, "Number of targets.")
+    .def("getNumberOfNeighbors", &NeighborLists<ParticleHelper::int_1d_view_type_in_gmls>::getNumberOfNeighborsHost, py::arg("target index"), "Get number of neighbors for target.")
+    .def("getNeighbor", &NeighborLists<ParticleHelper::int_1d_view_type_in_gmls>::getNeighborHost, py::arg("target index"), py::arg("local neighbor number"), "Get neighbor index from target index and local neighbor number.")
+    .def("getMaxNumNeighbors", &NeighborLists<ParticleHelper::int_1d_view_type_in_gmls>::getMaxNumNeighbors, "Get maximum number of neighbors over all neighborhoods.")
+    .def("getTotalNeighborsOverAllLists", &NeighborLists<ParticleHelper::int_1d_view_type_in_gmls>::getTotalNeighborsOverAllListsHost, "Get total storage size of all neighbor lists combined.");
 
     py::class_<KokkosParser>(m, "KokkosParser")
     .def(py::init<int,int,int,int,bool>(), py::arg("num_threads") = -1, py::arg("numa") = -1, py::arg("device") = -1, py::arg("ngpu") = -1, py::arg("print") = false);
